/*
 * ctimer.cpp
 * Julian Smart
 */

#include "stdafx.h"
#include "ctimer.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

static CMapWordToOb CTimerList;
UINT WINAPI CTimerProc(HWND hwnd, WORD, int idTimer, DWORD);

CTimer::CTimer(void)
{
  id = 0;
  
  milli = 0 ;
  lastMilli = -1 ;
}

CTimer::~CTimer(void)
{
  Stop();

  CTimerList.RemoveKey((WORD)id);
}

// Override me!
void CTimer::Notify(void)
{
}

int CTimer::Interval(void)
{
  return milli ;
}

BOOL CTimer::Start(int milliseconds, BOOL mode)
{
  oneShot = mode ;
  if (milliseconds < 0)
    milliseconds = lastMilli;

  if (milliseconds <= 0)
    return FALSE;

  lastMilli = milli = milliseconds;

  CTimerList.RemoveKey((WORD)id);

  TIMERPROC CTimerProcInst = (TIMERPROC) MakeProcInstance((FARPROC)CTimerProc,
                                          AfxGetInstance());

  id = SetTimer(NULL, (UINT)(id ? id : 1), (UINT)milliseconds, CTimerProcInst);
  if (id > 0)
  {
    CTimerList.SetAt((WORD)id, this);
    return TRUE;
  }
  else return FALSE;
}

void CTimer::Stop(void)
{
  if (id) {
    KillTimer(NULL, (UINT)id);
    CTimerList.RemoveKey((WORD)id);
  }
  id = 0 ;
  milli = 0 ;
}

UINT WINAPI CTimerProc(HWND hwnd, WORD, int idTimer, DWORD)
{
  CTimer *timer = NULL;
  if (!CTimerList.Lookup((WORD)idTimer, (CObject*&) timer) || !timer)
	  return 0;

  if (timer->id==0)
    return(0) ; // Avoid to process spurious timer events
  if (timer->oneShot)
    timer->Stop() ;
  timer->Notify();
  return 0;
}
